@@ -4,7 +4,7 @@
 
 START_EXTERN_C
 EXTERN_C const regexp_engine re2_engine;
-EXTERN_C void RE2_possible_match_range(REGEXP* rx, STRLEN len, SV **possible_min, SV **possible_max);
+EXTERN_C void RE2_possible_match_range(pTHX_ REGEXP* rx, STRLEN len, SV **possible_min, SV **possible_max);
 END_EXTERN_C
 
 extern unsigned re2_max_mem;
